/*
 * panel.c
 *
 *  Created on: 12/12/2015
 *      Author: JUANENRIQUE
 */

#include "aquariosCustos.h"

#define retLinea "\r\n"

extern uint8_t nFeeds;
extern uint8_t feedHour[MAXFEEDS];
extern uint8_t feedMin[MAXFEEDS];
extern uint8_t feedSteps[MAXFEEDS];
extern uint8_t feedToday[MAXFEEDS];
extern uint8_t feedNext;
extern uint8_t feedDiaActual;

extern float gblWaterTemp;
extern float gblWaterPH;
extern float gblWaterMinTemp,gblWaterMaxTemp;
extern float gblPHMin,gblPHMax;
extern float gblEnvLight;
extern float gblEnvLightMinThreshold;
extern uint8_t gblSwOnHour, gblSwOnMin, gblSwOffHour, gblSwOffMin;
extern uint8_t gblLightStatus;
extern uint8_t ds18s20Present;
extern uint8_t pHSensorPresent;

const char *estado[] = {"Apagado  ","Encendido"};  // Apagado has extra spaces to keep same length on each line

char *titulo     ="\f\x1B[0;37;40m\r\n                        AQUARIOS CUSTOS v%d.%d\r\n";
char *panSepa    ="##################################################################\r\n";
char *panFecha   ="##  Fecha: %02d/%02d/%04d           ||  Hora: %02d:%02d:%02d              ##\r\n";
char *panSep1    ="##------------------------------++------------------------------##\r\n";
char *panTemp    ="##  Temperatura Agua: %05.2f ºC  || pH Agua: %03.2f pH             ##\r\n";
char *panUTemp   ="##  Umbral > %5.2f a < %05.2f    || Umbral > %03.2f a < %03.2f       ##\r\n";
char *intLuz     ="##  Intensidad Luz: %05.2f[%04.2f]     Estado Luz: %s       ##\r\n";
char *intHLuz    ="##  Horario encendido: %02d.%02d    ||  Horario Apagado: %02d.%02d      ##\r\n";
char *feedND     ="##  Número de alimentaciones por día: %02d                        ##\r\n";
char *feedUlt    ="##  Ultima Alimentación:  %02d:%02d            Cantidad: %02d pasos   ##\r\n";
char *feedNxt    ="##  Próxima Alimentación: %02d:%02d            Cantidad: %02d pasos   ##\r\n";
char *txtEspera  ="[ENTER] para refrescar.\r\n";

char *menu1		 = "\r\nMenú:\r\n\r\n[t] Cambiar umbrales de temperatura\r\n[p] Cambiar umbrales de pH\r\n[l] Gestión de luz artificial\r\n[a] Gestión de alimentación\r\n\r\n[f] Establecer fecha actual\r\n\r\n";


char *txtTempMin ="\r\nIntroduzca rango mínimo de temperatura: ";
char *txtTempMax ="\r\nIntroduzca rango máximo de temperatura: ";

char *txtPHMin   ="\r\nIntroduzca rango mínimo de pH (min 0.00) [%03.2f]: ";
char *txtPHMax   ="\r\nIntroduzca rango máximo de pH (max 0.00) [%03.2f]: ";

char *txtAno = "\r\nIntroduzca año actual [%04d]:";
char *txtMes = "\r\nIntroduzca mes actual [%02d]:";
char *txtDia = "\r\nIntroduzca día actual [%02d]:";
char *txtHora= "\r\nIntroduzca hora atual [%02d]:";
char *txtMinutos = "\r\nIntroduzca minutos actuales [%02d]:";

char *txtLuzUmbral  ="\r\nIntroduzca umbral de luz a partir del cual no es necesaria luz artificial [%05.2f]: ";
char *txtSwOnHour	="\r\nHora a la que se encenderá la iluminación artificial [%02d]:";
char *txtSwOnMin	="\r\nMinutos a los que se encenderá la iluminación artificial [%02d]:";
char *txtSwOffHour	="\r\nHora a la que se apagará la iluminación artificial [%02d]:";
char *txtSwOffMin	="\r\nMinutos a los que se apagará la iluminación artificial [%02d]:";

char *txtFeedNum    ="\r\nNúmero de alimentaciones diaria [%02d]:";
char *txtFeedOnHour ="\r\nAlim. Número %02d. Hora de inicio:";
char *txtFeedOnMin  ="\r\nAlim. Número %02d. Minuto de inicio:";
char *txtFeedSteps	="\r\nAlim. Número %02d. Número de pasos de alimento:";
char *feedWarning   ="\r\n***********\r\n Atención las horas y minutos que introduzca deben ir\r\n por orden desde la 00:00 hasta las 23:59. Si se introducen desordenadas \r\nse perderá alguna alimentación.\r\n***********\r\n";
char *txtFeedAction ="\r\n-> Enviando alimentación %02d de %02d";


/* Warnings */
char *txtTempWarning = "\r\n\x1B[0;31;40m[AVISO] El umbral de temperatura está fuera de rango. Temperatura actual: %05.2f, Temp. mínima: %05.2f, Temp. máxima: %05.2f\x1B[0;37;40m\r\n";
char *txtTempSensorW = "\r\n\x1B[0;31;40m[AVISO] El sensor de temperatura definido no se ha encontrado\x1B[0;37;40m\r\n";
char *txtPHSensorW	 = "\r\n\x1B[0;31;40m[AVISO] El sensor de pH no da lecturas correctas\x1B[0;37;40m\r\n";
char *txtPHWarning   = "\r\n\x1B[0;31;40m[AVISO] El umbral de pH está fuera de rango. pH actual: %03.2f, pH. mínimo: %03.2f, pH. máximo: %03.2f\x1B[0;37;40m\r\n";

/** Get a number (with or withoug decimal point) vía serial port. We support basic commnads like enter or delete.
 *
 * @PortNum		Serial port on which we wait for keystrokes
 * @nChars		max. num or chars to receive, after that it's like a ENTER key press
 *
 * @return 		a float number. We support numbers with decimal point, and negative numbers.
 */
float GetSerialNumber(uint32_t PortNum, uint8_t nChars);

/* Functions */
void Panel_Print()
{
	uint16_t lclDay=0,lclMonth=0,lclYear=2015,lclHour=10,lclMinute=10,lclSec=10;

	lclHour 	= LPC_RTC->HOUR;
	lclMinute 	= LPC_RTC->MIN;
	lclSec 		= LPC_RTC->SEC;
	lclDay 		= LPC_RTC->DOM;
	lclMonth 	= LPC_RTC->MONTH;
	lclYear 	= LPC_RTC->YEAR;

	// printf menu
	Printf_Print(SERIALPORT,titulo,maxVersion,minVersion);
	Printf_Print(SERIALPORT,panSepa);
	Printf_Print(SERIALPORT,panFecha,lclDay,lclMonth,lclYear,lclHour,lclMinute,lclSec);
	Printf_Print(SERIALPORT,panSep1);
	Printf_Print(SERIALPORT,panTemp,(ds18s20Present==1?gblWaterTemp/100:-1),gblWaterPH);
	Printf_Print(SERIALPORT,panUTemp,gblWaterMinTemp,gblWaterMaxTemp,gblPHMin,gblPHMax);
	Printf_Print(SERIALPORT,panSep1);
	Printf_Print(SERIALPORT,intLuz,gblEnvLight,gblEnvLightMinThreshold,estado[gblLightStatus]);
	Printf_Print(SERIALPORT,intHLuz,gblSwOnHour,gblSwOnMin,gblSwOffHour,gblSwOffMin);
	Printf_Print(SERIALPORT,panSep1);
	Printf_Print(SERIALPORT,feedND,nFeeds);
	if (feedNext==0 && nFeeds > 1)	Printf_Print(SERIALPORT,feedUlt,0,0,0,0,0,0);
		else Printf_Print(SERIALPORT,feedUlt,feedHour[(feedNext==0?0:feedNext-1)],feedMin[(feedNext==0?0:feedNext-1)],feedSteps[(feedNext==0?0:feedNext-1)]);

	Printf_Print(SERIALPORT,feedNxt,feedHour[feedNext],feedMin[feedNext],feedSteps[feedNext]);
	Printf_Print(SERIALPORT,panSepa);
	Printf_Print(SERIALPORT,retLinea);
	Printf_Print(SERIALPORT,txtEspera);
	//Printf_Print(SERIALPORT,retLinea);
	Printf_Print(SERIALPORT,menu1);

	if (!ds18s20Present)
	{
		Printf_Print(SERIALPORT,retLinea);
		Printf_Print(SERIALPORT,txtTempSensorW);
	}

	if (!pHSensorPresent)
	{
		Printf_Print(SERIALPORT,retLinea);
		Printf_Print(SERIALPORT,txtPHSensorW);
	}

}

void UpdateTemp()
{

	Printf_Print(SERIALPORT,txtTempMin);
	gblWaterMinTemp = GetSerialNumber(SERIALPORT, 5);

	Printf_Print(SERIALPORT,retLinea);

	Printf_Print(SERIALPORT,txtTempMax);
	gblWaterMaxTemp = GetSerialNumber(SERIALPORT, 5);

	Panel_Print();

}

void UpdatePH()
{

	Printf_Print(SERIALPORT,txtPHMin,gblPHMin);
	gblPHMin = GetSerialNumber(SERIALPORT,4);

	Printf_Print(SERIALPORT,txtPHMax,gblPHMax);
	gblPHMax = GetSerialNumber(SERIALPORT,4);

	Panel_Print();

}

void UpdateFeed()
{

	uint8_t i=0;

	Printf_Print(SERIALPORT,txtFeedNum,nFeeds);
	nFeeds = GetSerialNumber(SERIALPORT,2);
	if (nFeeds > MAXFEEDS )	nFeeds = MAXFEEDS;

	// Warning
	Printf_Print(SERIALPORT,feedWarning);

	for(i=0;i<nFeeds;i++)
	{
		// Get each feed hour and min information
		Printf_Print(SERIALPORT, txtFeedOnHour, i);
		feedHour[i] = GetSerialNumber(SERIALPORT,2);
		Printf_Print(SERIALPORT, txtFeedOnMin, i);
		feedMin[i]  = GetSerialNumber(SERIALPORT,2);
		Printf_Print(SERIALPORT, txtFeedSteps,i);
		feedSteps[i] = GetSerialNumber(SERIALPORT,2);
		feedToday[i] = 0;
	}

	feedDiaActual = LPC_RTC->DOM;

	feedHour[nFeeds]=23;
	feedMin[nFeeds] =59;
	feedSteps[nFeeds]=0;

	// Search next feed to update screen
	for(i=0;((i<nFeeds) && ((LPC_RTC->HOUR*60+LPC_RTC->MIN)>(feedHour[i]*60+feedMin[i])));i++);
	// We got it or if we're at then end restart
	if (i==nFeeds)	feedNext=0; else feedNext=i;

	Panel_Print();

}

void UpdateDate()
{

	Printf_Print(SERIALPORT,txtAno,LPC_RTC->YEAR);
	LPC_RTC->YEAR = GetSerialNumber(SERIALPORT,4);

	Printf_Print(SERIALPORT,txtMes,LPC_RTC->MONTH);
	LPC_RTC->MONTH = (int)GetSerialNumber(SERIALPORT,2);

	Printf_Print(SERIALPORT,txtDia,LPC_RTC->DOM);
	LPC_RTC->DOM = (int)GetSerialNumber(SERIALPORT,2);

	Printf_Print(SERIALPORT,txtHora,LPC_RTC->HOUR);
	LPC_RTC->HOUR = (int)GetSerialNumber(SERIALPORT,2);

	Printf_Print(SERIALPORT,txtMinutos,LPC_RTC->MIN);
	LPC_RTC->MIN = (int)GetSerialNumber(SERIALPORT,2);
	Panel_Print();

}

void UpdateLight()
{

	Printf_Print(SERIALPORT,txtLuzUmbral,gblEnvLightMinThreshold);
	gblEnvLightMinThreshold = GetSerialNumber(SERIALPORT, 5);
	Printf_Print(SERIALPORT,retLinea);

	Printf_Print(SERIALPORT,txtSwOnHour,gblSwOnHour);
	gblSwOnHour = GetSerialNumber(SERIALPORT, 2);
	Printf_Print(SERIALPORT,retLinea);

	Printf_Print(SERIALPORT,txtSwOnMin,gblSwOnMin);
	gblSwOnMin = GetSerialNumber(SERIALPORT, 2);
	Printf_Print(SERIALPORT,retLinea);

	Printf_Print(SERIALPORT,txtSwOffHour,gblSwOffHour);
	gblSwOffHour = GetSerialNumber(SERIALPORT, 2);
	Printf_Print(SERIALPORT,retLinea);

	Printf_Print(SERIALPORT,txtSwOffMin,gblSwOffMin);
	gblSwOffMin = GetSerialNumber(SERIALPORT, 2);
	Printf_Print(SERIALPORT,retLinea);
	Panel_Print();
}

/* Vamos a chequear los umbrales de temp y ph, las horas, etc. */
void CompruebaUmbralesAcciones()
{
	uint8_t i=0;

	// Check temperature thresholds
	if (ds18s20Present)
	{
		if (((gblWaterTemp/100) < gblWaterMinTemp) || ((gblWaterTemp/100) > gblWaterMaxTemp))
		{
			// The water temp is outside the temp threshold
			Printf_Print(SERIALPORT,txtTempWarning, gblWaterTemp/100, gblWaterMinTemp, gblWaterMaxTemp);
		}
	}
	// Check pH thresholds
	if (pHSensorPresent)
	{
		if ((gblWaterPH < gblPHMin) || (gblWaterPH > gblPHMax))
		{
			// We're outside the predefined threshold
			Printf_Print(SERIALPORT,txtPHWarning, gblWaterPH, gblPHMin, gblPHMax);
		}
	}
	// Check artificial light status and change if needed
	if (((LPC_RTC->HOUR*60+LPC_RTC->MIN) >= (gblSwOnHour*60+gblSwOnMin)) && ((LPC_RTC->HOUR*60+LPC_RTC->MIN) <= (gblSwOffHour*60+gblSwOffMin)))
	{
		// Ok, it should be on, but do we have enough environmental light?
		if ( gblEnvLight < gblEnvLightMinThreshold )
		{
			// The enviromental light is not enough, switchon the lights!
			gblLightStatus = lights_Switch(1); // ON
		} else {
			// We have enough enviromental light switch it off and save the planet
			gblLightStatus = lights_Switch(0);
		}

	} else {
		// ey, we're on night time, let get our dreams come true, the lights must be off
		gblLightStatus = lights_Switch(0);
	}

	// Check if we're over the time of the next feed event
	if (((LPC_RTC->HOUR*60+LPC_RTC->MIN) >= (feedHour[feedNext]*60+feedMin[feedNext]))
			&&
			((LPC_RTC->HOUR*60+LPC_RTC->MIN) < (feedHour[feedNext+1]*60+feedMin[feedNext+1]))
			&&
				((feedToday[feedNext] == 0))   // Not feeded today
				)
	{
		// Ok, we're higger, so feed the fish!
		for (i=0;i<feedSteps[feedNext];i++)
		{
			Printf_Print(SERIALPORT,txtFeedAction,i+1,feedSteps[feedNext]);
			feeder_Switch(1); Utils_DelayMs(1000); feeder_Switch(0);
			Utils_DelayMs(1000);
		}
		feedToday[feedNext] = 1;   // Today they got their feed

		feedNext++; if (feedNext>=nFeeds) feedNext = 0;
		Panel_Print();    // Update panel to show next feed time
	}

	// Check if it is a new day
	if ((LPC_RTC->DOM != feedDiaActual))
	{	// Clear all feeds
		for (i=0; i<nFeeds; i++) feedToday[i] = 0;
		feedDiaActual = LPC_RTC->DOM;
	}


}

/* Get a number (decimal of float) from Serial Port, and returns a float number */
float GetSerialNumber(uint32_t PortNum, uint8_t nChars)
{

	char *lclTemp[10];
	uint8_t chrIdx=0;
	uint8_t loopExit = 1;
	char *readChar[5];  // Enough letters to be hold

	readChar[0]=0;	lclTemp[0]=0;

	// We just support 10 max Chars
	if (nChars>10) nChars=10;

	while ((chrIdx < nChars) && (*readChar[0] != '\n') && (loopExit == 1))
	{
		while (strcmp((char *)readChar,NULL)==0)
			Printf_Scan(PortNum,100, "%c", (char*)readChar);

		// Manage control chars
		if ((int)readChar[0] == 13) loopExit = 0; // He has finished
		else
			if ((int)readChar[0] == 127)
			{
				if (chrIdx > 0)	{ chrIdx-=1; lclTemp[chrIdx] = 0; }
			} else {
				strcat((char *)lclTemp,(char *)readChar); chrIdx++;
			}
//		Printf_Print(PortNum,"[Valor chrIdx: %i valor char: %i]\r\n",chrIdx,(int)readChar[0]);
		Printf_Print(PortNum,"%c",readChar[0]);

//		lclTemp[chrIdx++] = readChar[0];
		readChar[0] = 0;
	}
	lclTemp[chrIdx]=0;
	return Utils_Jatol((char*)lclTemp);
}
