// --- main.c ---

#include "aquariosCustos.h"


/* Priorities at which the tasks are created. */
#define STANDARD_TASK_PRIORITY				( tskIDLE_PRIORITY + 2 )
#define HIGH_TASK_PRIORITY					( tskIDLE_PRIORITY + 1 )

/* Feed Parameters */

uint8_t nFeeds = 0;
// We use +1 so we can always store the late time possible for comparisions
uint8_t feedHour[MAXFEEDS+1];
uint8_t feedMin[MAXFEEDS+1];
uint8_t feedSteps[MAXFEEDS+1];
uint8_t feedToday[MAXFEEDS];
uint8_t feedNext=0;
uint8_t feedDiaActual;  // Control if today we have already feeded the fishes

/* Global variable definitions */
float gblWaterTemp = 14.0f;
float gblWaterPH = 7.0f;
float gblWaterMinTemp=14.0f,gblWaterMaxTemp=22.0f;
float gblPHMin=0.0f,gblPHMax=7.6f;
float gblEnvLight = 0;
float gblEnvLightMinThreshold=0.45f;
uint8_t gblSwOnHour=9, gblSwOnMin=0, gblSwOffHour=23, gblSwOffMin=59;
uint8_t gblLightStatus = 0; // 0 Switch off - 1 Switch on

/* Global variable to check if the DS18s20 is present */
uint8_t ds18s20Present = 0;
uint8_t pHSensorPresent = 0;

/* Local functions */
static void prvShowPanel( void *pvParameters );
static void prvReadEnvLightPH( void *pvParameters );
static void prvReadTemp( void *pvParameters );
static void prvCheck( void *pvParameters );
void initDevices(void);
void InitTempSensor(void);

/*-----------------------------------------------------------*/

int main(void)
{

	// Initialize variables
	// initVars();

	// Initialize devices
	initDevices();

	// vamos a crear las siguientes tareas:

	// Tarea 1: Mostrar el panel con los datos actuales leidos
	xTaskCreate( prvShowPanel, ( signed char * ) "shwPanel", 1024, NULL, STANDARD_TASK_PRIORITY, NULL );

	// Tarea 2: Leer la luz de ambiente
	xTaskCreate( prvReadEnvLightPH, ( signed char *) "readEnvLghtPH", 1024, NULL, STANDARD_TASK_PRIORITY, NULL);

	// Tarea 3.0: Leer la temperatura
	xTaskCreate( prvReadTemp, ( signed char *) "readTemp", 1024, NULL, HIGH_TASK_PRIORITY, NULL);

	// Tarea 4: Comprobar umbrales y acciones
	xTaskCreate( prvCheck, ( signed char *) "CheckThrsld", 1024, NULL, HIGH_TASK_PRIORITY, NULL);

	vTaskStartScheduler();


	for( ;; );
}

void initDevices()
{

	// Initialize Serial human communication port
	Printf_Start(SERIALPORT,SERIALSPEED);
	if (USESERIALLOG == 1) Log_Start(LOGSERIALPORT,LOGSERIALSPEED);

	// Initialize Light sensor

	ADC_init(ADCLIGHT);  // Let's init AD0.0 to read light intense
	ADC_init(ADCPH);  // Let's init AD0.1 to read PH values

	InitTempSensor();

}
/*-----------------------------------------------------------*/

void InitTempSensor()
{

	// Initialize temperature sensor
		t_sensor[0].l = SENSORSERIALNO;

		ow_init(); //init ow
		if (ow_find_devices(t_sensor[0].l) == 1) ds18s20Present = 1;
		if (ds18s20Present == 1) Printf_Print(SERIALPORT,"Encontrado el sensor!\r\n");

}

/*-----------------------------------------------------------*/

static void prvShowPanel( void *pvParameters )
{

const portTickType xDelay = 100 / portTICK_RATE_MS;  // We give 100ms to other tasks while waiting for key on serial port

uint32_t	 tEspera = 0;

char *opcMenu[10];

	for( ;; )
	{
		Panel_Print(); // Show current panel status

		opcMenu[0]=NULL;
		while (opcMenu[0]==NULL)
		{
			Printf_Scan(SERIALPORT,100, "%c", (const char*) opcMenu);
			vTaskDelay(xDelay);
			tEspera++;

			if (tEspera > ((60*1000)/100))
			{
				Panel_Print();  // Refresh each 60 secs. (probably a bit more)
				tEspera = 0;
			}

		}

		tEspera = 0;
		opcMenu[1]=0;

		Printf_Print(3,"opcMENU=%s\r\n",opcMenu);

		if (strcmp((char *)opcMenu,"t")==0 || strcmp((char *)opcMenu,"T")==0)	UpdateTemp();
		if (strcmp((char *)opcMenu,"p")==0 || strcmp((char *)opcMenu,"P")==0)	UpdatePH();
		if (strcmp((char *)opcMenu,"f")==0 || strcmp((char *)opcMenu,"F")==0)	UpdateDate();
		if (strcmp((char *)opcMenu,"l")==0 || strcmp((char *)opcMenu,"L")==0)	UpdateLight();
		if (strcmp((char *)opcMenu,"a")==0 || strcmp((char *)opcMenu,"A")==0)	UpdateFeed();

	}

}

/*-----------------------------------------------------------*/

static void prvReadEnvLightPH( void *pvParameters )
{
portTickType xNextWakeTime;
const float maxAdcBits = 4095.0f; // 12 bits = 4096
const float maxVolts = 3.3f;      // Max ADC voltage input
const float voltsPerBit = (maxVolts / maxAdcBits);

	xNextWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		gblEnvLight = ADC_read(0) * voltsPerBit;
		gblWaterPH = 3.5f * ADC_read(1) * voltsPerBit;
		if (gblWaterPH == 0 || gblWaterPH >=16)
		{
			if (pHSensorPresent) {
				pHSensorPresent = 0;
				Panel_Print();
			}
		} else pHSensorPresent = 1;

		vTaskDelayUntil( &xNextWakeTime, envlightREAD_FREQUENCY_MS );
	}
}
/*-----------------------------------------------------------*/

static void prvReadTemp( void *pvParameters )
{
portTickType xNextWakeTime;
uint16_t lclTemp;

	xNextWakeTime = xTaskGetTickCount();

	for ( ;; )
	{
		if (ds18s20Present == 1)
		{
			// We just support 1 sensor
			if (ow_find_devices(t_sensor[0].l) == 1)  // Check if the Temp Sensor is already conn
			{
				if (ow_read_temp(&lclTemp, &t_sensor[0])) {
				} else {
					gblWaterTemp = (lclTemp * 5); //read uint to int
				}
			} else {ds18s20Present = 0; 	Panel_Print();}
		} else {
			// Lets try to reconnect the Temp Sensor
			InitTempSensor();
			Panel_Print();
		}
		vTaskDelayUntil( &xNextWakeTime, tempREAD_FREQUENCY_MS);
	}

}
/*-----------------------------------------------------------*/
static void prvCheck( void *pvParameters )
{
	portTickType xNextWakeTime;

	xNextWakeTime = xTaskGetTickCount();
	for ( ;; )
	{
		CompruebaUmbralesAcciones();
		vTaskDelayUntil ( &xNextWakeTime, checkREAD_FREQUENCY_MS);
	}
}

/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
