/**
 * GPIO management module
 * bunch of functions to manage and configure GPIO Ports on LPC.
 *
 * Juan E. Gómez Pérez
 * UOC - TFG Sistemas Embebidos
 *
 * revision 1.0 date 20/10/2015
 */

#ifndef GPIO_H_
#define GPIO_H_

/*
 * Function definitions
 */

/** Raises a GPIO configured pin during 1 ms, and lowers it again after 1 ms. It's used to reset modules like wifly.
 *
 *
 * @gpio_Port		Port used on GPIO. Check manual.
 * @gpio_Pin		PIN selected from this port. Check manual.
 *
 * @return 			none
 */
void	GPIO_send_reset_signal(uint8_t gpio_Port, uint8_t gpio_Pin);

/** Initialize ADC converter and selected ADC Port
 *
 *
 * @adc_Port	ADC Port number to be initialized
 *
 * @return 		none
 */
void ADC_init(uint8_t adc_Port);

/** Read a data from the ADC Pin, wait for conversion and send back to the requester
 *
 *
 * @adc_Port	ADC Port to be read
 *
 * @return 		INT16 of the value read
 */
uint16_t ADC_read(uint8_t adc_Port);

/** Sets a GPIO pin in a high state *
 *
 * @gpio_Port	Port to be used
 * @gpio_Pin	Pin of port to be raised
 *
 * @return 		none
 */
void GPIO_pin_on(uint8_t gpio_Port, uint8_t gpio_Pin);

/** Sets a GPIO pin in a low state *
 *
 * @gpio_Port	Port to be used
 * @gpio_Pin	Pin of port to be raised
 *
 * @return 		none
 */
void GPIO_pin_off(uint8_t gpio_Port, uint8_t gpio_Pin);

#endif /* GPIO_H_ */
