/**
 * Manage strings on LPC UART ports
 *
 * Juan E. Gómez Pérez
 * UOC - TFG Sistemas Embebidos
 *
 * revision 1.0 date 20/10/2015
 */

#ifndef SERIAL_LIB_INC_PRINTF_HANDLER_H_
#define SERIAL_LIB_INC_PRINTF_HANDLER_H_

/** Initialize PRINTF module against an UART port
 *
 * @PortNum		UART port number to send debug data
 * @baudrate	UART baudrate on which we will send debug data
 *
 * @return 		1 if successful, 0 if error.
 */
uint32_t Printf_Start( uint32_t PortNum, uint32_t baudrate );

/** printf a formated string to a preinitialized UART port
 *
 * @PortNum		UART port number to send debug data
 * @cadena		string to be sent to the UART
 * @...			format, or additional strings to be sent to the UART
 *
 * @return 		void
 */
void Printf_Print( uint32_t PortNum, const char *cadena,  ...);

/** scanf an RX string from the UART on defined port, with a definable wait time
 *
 * @PortNum		UART port number to receive data
 * @espera		ms to Wait for data to come
 * @formato 	format string to receive the data and transform in a managed way
 * @caracteres	string on which we will return the received data
 *
 * @return 		length of the received data
 */
uint32_t Printf_Scan( uint32_t PortNum, uint32_t Espera, const char *formato, const char *cadena);

/** scanf an RX string from the UART on defined port, with a definable wait time. But in a raw manner.
 *
 * @PortNum		UART port number to receive data
 * @espera		ms to Wait for data to come
 * @caracteres	string on which we will return the received data
 *
 * @return 		length of the received data
 */
uint32_t Printf_ScanCompleto( uint32_t PortNum, uint32_t Espera, char *cadena);

#endif /* SERIAL_LIB_INC_PRINTF_HANDLER_H_ */
