// --- arpalab.c ---


#include "FreeRTOS.h"
#include "LPC17xx.h"

#include <semphr.h>

#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include "log.h"
#include "wifly.h"
#include "utils.h"

#define arpDebug	1

uint8_t	arpalab_Initialized = 0;
xSemaphoreHandle	xArpaBinarySem;
static char arpabuffer_RX[1024];


int8_t arpalab_initialize(void)
{

	if (arpalab_Initialized != 1)
	{
		vSemaphoreCreateBinary(xArpaBinarySem);
		if( Wifly_HTTPInit(80, "arpalab.appspot.com")!=1) { return -1; } // Initialize ARP@LAB
		arpalab_Initialized = 1;
	} else return -1;

	return arpalab_Initialized;

}

int32_t arpalab_getdata(void)
{
	uint32_t arpaDato = 0;
	char  url[256];
	char* cadenaBuscada="\r\n\r\n"; // look for line which separates Headers and Data
	char* cadenaSplit="*CLOS";

	// Check if arpalab mutex has been initialized, if not do it
	if (arpalab_Initialized != 1) arpalab_initialize();

	xSemaphoreTake(xArpaBinarySem, portMAX_DELAY);   // Wait for the ARP@ module to be free... forever

	sprintf(url,"http://arpalab.appspot.com/rest/feed\n\n");
	if (arpDebug) Log_Log("[arp@Lab]GETDATA: Gending URL Data: %s",url);
	Wifly_HTTPGet("arpalab.appspot.com 80",url,arpabuffer_RX);

	if (arpDebug) Log_Log("[arp@lab]GETDATA: Received GET answer: %s",arpabuffer_RX);

	char* salida = strstr(arpabuffer_RX,cadenaBuscada) + 4;
	if (arpDebug) Log_Log("[arp@Lab]GETDATA: salida: %s",salida);
	// Trunc at 40 as maximum
	sprintf(arpabuffer_RX,"%s",salida);
	strtok(arpabuffer_RX,cadenaSplit);

	arpaDato =Utils_Jatoi(arpabuffer_RX);

	// Give the sempahore
	xSemaphoreGive(xArpaBinarySem);

	return arpaDato;

}

int32_t arpalab_setdata(uint32_t NID, char *lIP, char *lAPP, float lData)
{
	uint32_t http_result_code = 0;
	char* cadenaBuscada="HTTP/1.0"; // look for line which separates Headers and Data
	char* cadenaSplit=" ";
	char  url[256];

	// Check if arpalab mutex has been initialized, if not do it
	if (arpalab_Initialized != 1) arpalab_initialize();

	xSemaphoreTake(xArpaBinarySem, portMAX_DELAY);   // Wait for the ARP@ module to be free... forever

	sprintf(url,"http://arpalab.appspot.com/rest/stats?nid=%u&ip=%s&app=%s&data=%3.2f\n\n",NID,lIP,lAPP,lData);
	if (arpDebug) Log_Log("[arp@Lab]SETDATA: Sending URL Data: [%s]",url);
	Wifly_HTTPGet("arpalab.appspot.com 80",url,arpabuffer_RX);

	if (arpDebug) Log_Log("[arp@Lab]SETDATA: Received GET Data: %s",arpabuffer_RX);

	char* salida = strstr(arpabuffer_RX,cadenaBuscada) + 9;
	// Trunc at 40 as maximum
	sprintf(arpabuffer_RX,"%s",salida);
	strtok(arpabuffer_RX,cadenaSplit);
	http_result_code = Utils_Jatoi(arpabuffer_RX);

	// Give the sempahore
	xSemaphoreGive(xArpaBinarySem);

	return http_result_code;

}
