// --- printf.c ---

#include "LPC17xx.h"
#include "FreeRTOS.h"

#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include <task.h>
#include <semphr.h>
#include <queue.h>

#include "type.h"
#include "uart.h"
#include "utils.h"
#include "printf.h"

/* Reference UART library buffers and byte count */
extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];
extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[BUFSIZE];
extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];
extern void UARTSend( uint32_t portNum, uint8_t *BufferPtr, uint32_t Length );

static xSemaphoreHandle xMutexSerialPrint[4];
static xSemaphoreHandle xMutexSerialScanf[4];
static uint8_t InitializedSerialPort[4]={0,0,0,0};

#define MAXSCANFWAITTIME	0x4FFFF /* Wait 5 sec as a maximum */
#define pfMAXBUFFSIZE		1024

/* Define local functions */

uint32_t Printf_ReceiveSerialText(uint32_t PortNum, uint32_t Espera, char *cadena);

uint32_t Printf_Start( uint32_t PortNum, uint32_t baudrate )
{

	/* check if the Serial Port has already been initialized */
	if (InitializedSerialPort[PortNum] == 0)
	{
		InitializedSerialPort[PortNum] = 1;
		// Initialize the Mutex used for exclusion on serialprintf
		xMutexSerialPrint[PortNum] = xSemaphoreCreateMutex();
		xMutexSerialScanf[PortNum] = xSemaphoreCreateMutex();
		UARTInit(PortNum, baudrate);

		switch(PortNum){
			case 0:
				UART0Count=0;
				break;
			case 1:
				UART1Count=0;
				break;
			case 3:
				UART3Count=0;
				break;
			default:
				break;
		}
		return 1;
	} else return 0;

}

void Printf_Print( uint32_t PortNum, const char *cadena,  ...)
{

	char buffer[256];
	va_list argumentos;

	xSemaphoreTake( xMutexSerialPrint[PortNum], portMAX_DELAY );
	{
		va_start(argumentos,cadena);
		vsprintf(buffer,cadena,argumentos);
		switch(PortNum){
			case 0:
				UART0Count=0;
				break;
			case 1:
				UART1Count=0;
				break;
			case 3:
				UART3Count=0;
				break;
			default:
				break;
		}
		UARTSend(PortNum, (uint8_t *)buffer , strlen(buffer) );
		va_end(argumentos);
		xSemaphoreGive(xMutexSerialPrint[PortNum]);
	}
}

uint32_t Printf_Scan( uint32_t PortNum,uint32_t Espera, const char *formato, const char *cadena)
{
		char pfsBUFFER[pfMAXBUFFSIZE];
		uint32_t caracteres = 0;
		caracteres = Printf_ReceiveSerialText(PortNum, Espera, pfsBUFFER);
		sscanf(pfsBUFFER,formato,cadena);
		return caracteres;
}

uint32_t Printf_ScanCompleto( uint32_t PortNum,uint32_t Espera, char *cadena)
{
		return Printf_ReceiveSerialText(PortNum, Espera, cadena);
}

uint32_t Printf_ReceiveSerialText(uint32_t PortNum, uint32_t Espera, char *cadena)
{
	uint32_t indice = 0;
	uint32_t indice1 = 0;

	xSemaphoreTake( xMutexSerialScanf[PortNum], portMAX_DELAY );
	{
		switch(PortNum)
		{
			case 0:
//				UART0Count = 0; /* Prepare to start receiving */
				/* Enable Serial NVIC interrupts and wait some time */
				LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;/* Re-enable RBR */
				Utils_DelayMs(Espera); /* Wait */
				/* Let's wait for all the chars to arrive */
				do{
					indice = indice1;
					Utils_DelayMs(10);
					indice1 = UART0Count;
				}while(indice != indice1);
				LPC_UART0->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
				strncpy(cadena,(char *)UART0Buffer,indice1); cadena[indice1]='\0'; // End string
				break;
			case 1:
				UART1Count = 0; /* Prepare to start receiving */
				/* Enable Serial NVIC interrupts and wait some time */
				LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;/* Re-enable RBR */
				Utils_DelayMs(Espera); /* Wait for 2 seconds */
				/* Let's wait for all the chars to arrive */
				do{
					indice = indice1;
					Utils_DelayMs(10);
					indice1 = UART1Count;
				}while(indice != indice1);
				LPC_UART1->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
				strncpy(cadena,(char *)UART1Buffer,indice1); cadena[indice1]='\0'; // End string
				break;
//			case 2:
//				UART2Count = 0; /* Prepare to start receiving */
				/* Enable Serial NVIC interrupts and wait some time */
//				LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR;/* Re-enable RBR */
//				delayMs(Espera); /* Wait for 2 seconds */
				/* Let's wait for all the chars to arrive */
//				do{
//					indice = indice1;
//					delayMs(10);
//					indice1 = UART2Count;
//				}while(indice != indice1);
//				LPC_UART0->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
//				strncpy(cadena,(char *)UART2Buffer,indice1); cadena[indice1]='\0'; // End string
// 				UART2Count=0;
//				break;
			case 3:
				UART3Count = 0; /* Prepare to start receiving */
				/* Enable Serial NVIC interrupts and wait some time */
				LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;/* Re-enable RBR */
				Utils_DelayMs(Espera); /* Wait for 2 seconds */
				/* Let's wait for all the chars to arrive */
				do{
					indice = indice1;
					Utils_DelayMs(10);
					indice1 = UART3Count;
				}while(indice != indice1);
				LPC_UART3->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
				strncpy(cadena,(char *)UART3Buffer,indice1); cadena[indice1]='\0'; // End string
				break;
			default:
				break;

		}
		xSemaphoreGive(xMutexSerialScanf[PortNum]);
	}

	return indice1;

}
